# Copyright (c) 2023, Texas Instruments Incorporated
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:

#  Redistributions of source code must retain the above copyright
#  notice, this list of conditions and the following disclaimer.

#  Redistributions in binary form must reproduce the above copyright
#  notice, this list of conditions and the following disclaimer in the
#  documentation and/or other materials provided with the distribution.

#  Neither the name of Texas Instruments Incorporated nor the names of
#  its contributors may be used to endorse or promote products derived
#  from this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Author: Thomas
Date: 2022/03/16
Version: 1.0
Description: GUI for chamberless smoke detector signal response display and data logging. 
"""

# TODO:
# (1) GUI size optimization. Ex. reduce # of library import 
# (2) Reference signal input/display(?) when available. 

"""
# Useful commands: 
# To create .exe file, run following command. 
# --icon=myicon.ico for user icon, skip if none.
# pyinstaller --onefile --icon=myicon.ico [script_name_to_be_converted.py]
# pyinstaller --onefile --icon=TI_logo.ico sasi_smoke_detector_m0l1306.pyw
# if too slow, use one-directory. 
# pyinstaller --onedir --icon=myicon.ico [script_name_to_be_converted.py]
# pyinstaller --onedir sasi_smoke_detector_m0l1306.py   (preferred; when complete, check note 2)
# pyinstaller --onedir sasi_smoke_detector_m0l1306.pyw
# pyinstaller --onedir --icon=TI_logo.ico sasi_smoke_detector_m0l1306.py
# Note: (1) to not display console terminal, save file to .pyw
#       (2) after GUI creation, copy and paste the "TI_logo.png" file into the .ext file directory. Or else GUI will not open; need to automate this process in future. 
"""
#%%
import os
import datetime as dt
import time
import numpy as np
import matplotlib.pyplot as plt
# import pandas as pd
import serial
from serial.tools.list_ports import comports
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
import tkinter as tk
import tkinter.font as TkFont
import threading
# from multiprocessing import Process
from matplotlib.ticker import FormatStrFormatter

#%%
############################## Setup UART ##############################
def get_com_port_id(filter_field):
    ports = comports()
    comPort = []
    for i,p in enumerate(ports):
        t = p[1].split(" (")
        # print(t)
        if(np.isin(t[0], filter_field)):
            comPort.append(p[0]) # print(p[0])
    if(len(comPort)==0):
        comPort = ['']
    return comPort

def open_serial_port(c): 
    # Setup serial port
    ser = serial.Serial()
    ser.port = c
    ser.baudrate = 115200
    ser.bytesize = serial.EIGHTBITS #number of bits per bytes
    ser.parity = serial.PARITY_NONE #set parity check: no parity
    ser.stopbits = serial.STOPBITS_ONE #number of stop bits
    ser.timeout = 1 #timeout block read
    ser.xonxoff = False   #disable software flow control
    ser.rtscts = False    #disable hardware (RTS/CTS) flow control
    ser.dsrdtr = False    #disable hardware (DSR/DTR) flow control
    ser.writeTimeout = 1  #timeout for write
    ser.readTimeout = 1  #timeout for read
    return ser
##########################################################################

#### save data ####
# def save_data(df, name, fragment=1):
#     # Prepare capture directory
#     today_date = dt.datetime.now().strftime("%Y%m%d")
#     PROJECT_ROOT_DIR = "."; BASE_DIRECTORY = os.path.join(PROJECT_ROOT_DIR,"capture",str(today_date))
#     if not os.path.isdir(BASE_DIRECTORY):
#         os.makedirs(BASE_DIRECTORY) 

#     if(fragment==1):
#         s = 0
#         while(os.path.exists( BASE_DIRECTORY + "\\" + str(today_date) + "_" + name + "_" + str(s) + ".xlsx" )): s+=1
#     name = str(today_date) + "_" + name + "_" +str(s)
#     f1 = os.path.join(BASE_DIRECTORY, "{}.xlsx".format(name))
#     df.to_excel(f1, engine="xlsxwriter")

def save_data_txt(df, dflabel, name, fragment=0):
    # Prepare capture directory
    today_date = dt.datetime.now().strftime("%Y%m%d")
    PROJECT_ROOT_DIR = "."; BASE_DIRECTORY = os.path.join(PROJECT_ROOT_DIR,"capture",str(today_date))
    if not os.path.isdir(BASE_DIRECTORY):
        os.makedirs(BASE_DIRECTORY) 
    
    # get next new number
    s = 0
    while(os.path.exists(BASE_DIRECTORY + "\\" + str(today_date) + "_" + name + "_" + str(s) + ".txt")): 
        s+=1 # create a new file
    
    if(fragment==1):
        writeLabel = 1 # write a label since file doesn't exist
    else:
        if(s!=0):
            s-=1 # append data to latest file
        writeLabel = 0 # file exist; don't write label

    fname = str(today_date) + "_" + name + "_" +str(s)
    f1 = os.path.join(BASE_DIRECTORY, "{}.txt".format(fname))
    with open(f1, "a") as file:
        # Write label
        if(writeLabel == 1):
            for line in dflabel: # Write elements in each entry
                s = str(line)
                file.write("%s\t" % s.ljust(15))
            file.write("\n") # Newline per entry
        
        # Write data
        for dloc in df:
            for line in dloc: 
                s = str(line)
                file.write("%s\t" % s.ljust(15))
            file.write("\n") 
    
        
def save_fig(fig_handle, name, tight_layout=True, resolution=300, fragment=1):
    today_date = dt.datetime.now().strftime("%Y%m%d")
    PROJECT_ROOT_DIR = ".";  BASE_DIRECTORY = os.path.join(PROJECT_ROOT_DIR,"capture",str(today_date))
    if not os.path.isdir(BASE_DIRECTORY):
        os.makedirs(BASE_DIRECTORY) 

    if(fragment==1):
        s = 0
        while(os.path.exists( BASE_DIRECTORY + "\\" + str(today_date) + "_" + name + "_" + str(s) + ".png" )): s+=1
    name = str(today_date) + "_" + name + "_" +str(s)
    path = os.path.join(BASE_DIRECTORY, "{}.png".format(name))
    print("Saving figure ---> ", name)
    if tight_layout: fig_handle.tight_layout()
    fig_handle.savefig(path, format="png", dpi=resolution)

#### readData ####
Nrun = 0 # tracks number of runs
x1plot, y1plot, x2plot, y2plot, x3plot, y3plot, x4plot, y4plot = [], [], [], [], [], [], [], [] # for plotting, statistic calculation
NpacketLoss = 0 # tracks number of packet loss
flag_prev = 0
flag_timestamp = ''

def readData_init():
    global Nrun, Nplot, NpacketLoss, dd_all, headers
    global x1plot, y1plot, x2plot, y2plot, x3plot, y3plot, x4plot, y4plot
    # global AFTER_REF

    Nrun = 0 # tracks number of runs
    Nplot = 1 # tracks d[1]=1; start plotting when 2nd 1 detected; 0: start at next d[1]=1; 1: start immediately when received d[1]=1
    x1plot, y1plot, x2plot, y2plot, x3plot, y3plot, x4plot, y4plot = [], [], [], [], [], [], [], [] # for plotting, statistic calculation
    
    headers = ["Time_RX","Time_MCU_s","State","LED_mode","Pattern","SMOKE_FLAG","N_VCM_PULSES","N_VSIG_PULSES","N_PATTERN","N_PULSES",
               "DUTY_CYCLE","DEMOD_PERIOD","INTRST","MOD_PERIOD","Temperature_deg","Humidity_rh","Humidity_rh","Dp_um","MC_ugPerm3"]
    
    dd_all = []
    

def plot_stat(x_name, x_in):
    if(x_name == "Blue LED"):
        z = x_name + "\n" + \
            "N: {}".format(len(x_in)) + "\n" + \
            "Val: {:.3f}".format(x_in[-1]) + "\n" + \
            "Min: {:.3f}".format(np.min(x_in)) + "\n" + \
            "Max: {:.3f}".format(np.max(x_in)) + "\n" + \
            "Avg: {:.3f}".format(np.mean(x_in)) + "\n" + \
            "Std: {:.3f}".format(np.std(x_in))
        stat_label1.config(text=z)
    
    if(x_name == "IR LED"):
        z = x_name + "\n" + \
            "N: {}".format(len(x_in)) + "\n" + \
            "Val: {:.3f}".format(x_in[-1]) + "\n" + \
            "Min: {:.3f}".format(np.min(x_in)) + "\n" + \
            "Max: {:.3f}".format(np.max(x_in)) + "\n" + \
            "Avg: {:.3f}".format(np.mean(x_in)) + "\n" + \
            "Std: {:.3f}".format(np.std(x_in))
        stat_label2.config(text=z)

def plot_smoke_stat(x_name, x_th, x_in):
    if(x_name == "Blue LED"):
        if(x_in >= x_th):
            z = 'Blue: no smoke'
        else:
            z = 'Blue: smoke detected'
        smoke_label_blue.config(text=z)
        
    if(x_name == "IR LED"):
        if(x_in >= x_th):
            z = 'IR: no smoke'
        else:
            z = 'IR: smoke detected'
        smoke_label_ir.config(text=z)        

def plot_smoke_alarm(flag, timestamp):
    global flag_prev, flag_timestamp

    if(flag == 0):
        z = 'No smoke'
        flag_timestamp=''
    else:
        z = 'Smoke detected at'
    
    if ((flag_prev==0) & (flag==1)):
        flag_timestamp = timestamp

    flag_prev = flag
    z = z+'\n'+flag_timestamp
    smoke_alarm_label.config(text=z)
    
    
def plot_aq_stat(dp1, mc1):

    z = "dp:\t {:.3f} um".format(dp1) + "\n" + \
        "MassCon:\t {:.3f} mg/m\u00b3".format(mc1)
    # if(mc1 > 30):
    #     z = "dp: {:.3f}um".format(dp1) + "\n" + \
    #         "MassCon: >30mg/m\u00b3"
    # else:
    #     z = "dp: {:.3f}um".format(dp1) + "\n" + \
    #         "MassCon: {:.3f}mg/m\u00b3".format(mc1)
    aq_label.config(text=z)
    
## Reading Serial Data from M0L1306
def readData(NrunSave=100):
    # Global variables
    global Nrun, Nplot, NpacketLoss, dd_all, headers, startTime # AFTER_REF
    global x1plot, y1plot, x2plot, y2plot, x3plot, y3plot, x4plot, y4plot
    
    # Read data from MCU. Skip last newline character: \n
    rx_serial = ser1.readline().decode(errors='ignore')
    # print(rx_serial)
    dlist = rx_serial.split(" ")
    
    # print(dlist)
    timeNow = dt.datetime.now().strftime('%H:%M:%S.%f')[:-4]

    if((dlist[0]=="M0L1306") & (dlist[-1]=="\n")): 
        dlist = dlist[0:-1] # drop "\n"
        Nrun = Nrun+1 # Update run counter
        
        dlist[0] = 0 # Replace [0] with received time in later code 
        if(Nrun==1):
            startTime = dt.datetime.now().timestamp()
            timeSinceStartSec = 0
        else:
            timeSinceStartSec = dt.datetime.now().timestamp() - startTime
        
        # Convert received data from string to numbers
        # Prepare header and offset list
        header1 = ["Time_RX","Tick","State","SMOKE_FLAG","N_VSIG_PULSES","N_PULSES","Temperature_deg","Humidity_rh","Dp_um","MC_ugPerm3",
                   "VSIG_IR_BASE","VSIG_BL_BASE","VSIG_IR_AV","VSIG_BL_AV","VSIG_IR_SS","VSIG_BL_SS","VSIG_IR_TC","VSIG_BL_TC","VSIG_IR_MV","VSIG_BL_MV"]
        
        dd = [round(float(a), 4) for e,a in enumerate(dlist[0:len(header1)])] # convert to floating points
        dd[header1.index("Time_RX")] = timeNow # Replace with packet received time
        dd[header1.index("Tick")] = dd[header1.index("Tick")] # MCU system tick

        # Store RAW data if available
        N_VSIG_PULSES_MAX = 320 
        SASI_N_PATTERN_MAX = 3
        if(any(ele in ["RAW_ALL"] for ele in dlist)):
            raw = [0 for i in range(N_VSIG_PULSES_MAX*SASI_N_PATTERN_MAX)] # initialize all to 0
            for e,a in enumerate(dlist[dlist.index("RAW_ALL")+1:]):
                raw[e] = int(a)
            dd.extend(raw)

            header2 = ["VSIG_"+str(i) for i in range(N_VSIG_PULSES_MAX*SASI_N_PATTERN_MAX)] # hard code to support up to 320 pulses
            packetLen = len(header1) + int(dd[header1.index("N_PULSES")]) + 1 # "1" is for "RAW_ALL" term
        else:
            header2 = []
            packetLen = len(header1)

        headers = header1 + header2

        if(len(dlist)!=packetLen):
            NpacketLoss = NpacketLoss+1
            print("Sample size not matched ------------------------>", NpacketLoss) 
            Nrun = Nrun-1
        else:
            # Ready to display data
            vsig_ir = dd[headers.index("VSIG_IR_AV")] + dd[headers.index("VSIG_IR_BASE")]
            vsig_bl = dd[headers.index("VSIG_BL_AV")] + dd[headers.index("VSIG_BL_BASE")]
            flag_smoke_alarm = dd[headers.index("SMOKE_FLAG")]
            dp = dd[headers.index("Dp_um")]
            mc = dd[headers.index("MC_ugPerm3")]/1000 # convert to mg/m^3
            
            if(stopEvent.is_set()!=1):
                print("{}\t  {:d}\t  {:5.4f}\t  {:5.4f}\t  {:d}\t\t {:2.3f}\t {:.3f}".format(timeNow, Nrun, vsig_ir, vsig_bl, int(flag_smoke_alarm), dp, mc/1000))
            
            # Store data
            dd_all.append(dd)
            if(Nrun == 1):
                save_data_txt([dd], headers, filename, fragment=1) # create a new file and store the first entry
                dd_all=[]
            elif((Nrun % NrunSave) == 0):
                save_data_txt(dd_all, headers, filename, fragment=0) # store multi-entries
                dd_all=[]	

            ### GUI display ###
            NsampleDisplay = 300 # adjust display size
            if(len(x1plot) < NsampleDisplay): 
                x1plot = np.append(x1plot, np.float64(timeSinceStartSec)) # Nrun
                y1plot = np.append(y1plot, np.float64(vsig_bl))
                x2plot = np.append(x2plot, np.float64(timeSinceStartSec))
                y2plot = np.append(y2plot, np.float64(vsig_ir))
                
                x3plot = np.append(x3plot, np.float64(timeSinceStartSec))
                y3plot = np.append(y3plot, np.float64(dp))
                x4plot = np.append(x4plot, np.float64(timeSinceStartSec))
                y4plot = np.append(y4plot, np.float64(mc))
            else:
                x1plot[0:NsampleDisplay-1] = x1plot[1:NsampleDisplay]; x1plot[NsampleDisplay-1] = np.float64(timeSinceStartSec) # x axis
                y1plot[0:NsampleDisplay-1] = y1plot[1:NsampleDisplay]; y1plot[NsampleDisplay-1] = np.float64(vsig_bl) # line 1
                
                x2plot[0:NsampleDisplay-1] = x2plot[1:NsampleDisplay]; x2plot[NsampleDisplay-1] = np.float64(timeSinceStartSec) # x axis
                y2plot[0:NsampleDisplay-1] = y2plot[1:NsampleDisplay]; y2plot[NsampleDisplay-1] = np.float64(vsig_ir) # line 2

                x3plot[0:NsampleDisplay-1] = x3plot[1:NsampleDisplay]; x3plot[NsampleDisplay-1] = np.float64(timeSinceStartSec) # x axis
                y3plot[0:NsampleDisplay-1] = y3plot[1:NsampleDisplay]; y3plot[NsampleDisplay-1] = np.float64(dp) # line 3

                x4plot[0:NsampleDisplay-1] = x4plot[1:NsampleDisplay]; x4plot[NsampleDisplay-1] = np.float64(timeSinceStartSec) # x axis
                y4plot[0:NsampleDisplay-1] = y4plot[1:NsampleDisplay]; y4plot[NsampleDisplay-1] = np.float64(mc) # line 4

            lines1.set_xdata(x1plot); lines1.set_ydata(y1plot) # plot BL data
            lines2.set_xdata(x2plot); lines2.set_ydata(y2plot) # plot IR data
            lines3.set_xdata(x3plot); lines3.set_ydata(y3plot) # plot dp data
            lines4.set_xdata(x4plot); lines4.set_ydata(y4plot) # plot mc data
    
            plot_stat("Blue LED", y1plot) # plot stat
            plot_stat("IR LED", y2plot) # plot stat
            plot_smoke_alarm(flag_smoke_alarm, timeNow)
            plot_aq_stat(dp, mc)
            
            # #if(flag_comp1_p==1):
            # #	z = 'ALC ON'
            # #else:
            # #	z = 'ALC OFF' 
            # #ALC_label.config(text=z)
            # #fig1.gca().relim()
            # fig1.gca().autoscale_view()

            ax1.relim(); ax1.autoscale_view(); 
            ax11.relim(); ax11.autoscale_view()
            ax2.relim(); ax2.autoscale_view(); 
            ax22.relim(); ax22.autoscale_view(); 
            fig1.gca().relim(); fig1.gca().autoscale_view()
            
            canvas.draw()
    # AFTER_REF = root.after(200, readData) # start next after 200ms TTedit


def plotter():
    while True:
        readData()
        if(stopEvent.is_set()):
            break
        time.sleep(0.1)
    # print("Read data stopped")
    
    
############################## Setup GUI ##############################
ser1 = []
cond = False

SASI_WR_TYPE_OP = 0x10
SASI_WR_TYPE_THIR = 0x11
SASI_WR_TYPE_THBLUE = 0x12
# SASI_WR_TYPE_NUMPULSE = 1

SASI_WR_OP_START = 1
SASI_WR_OP_STOP = 0

def serial_write(reg, val, delay): 
    global ser1
    txData = [reg, (val & 0xFF), ((val >> 8) & 0xFF), ((val >> 16) & 0xFF), ((val >> 24) & 0xFF)]
    ser1.write(txData)
    time.sleep(delay) # seconds

def serial_read(reg):
    global ser1
    serial_write(reg, 0, 0)
    rxData = ser1.readline().decode('utf-8').split(',')
    rxData = np.int(rxData[1])
    return rxData

def user_entered_filename():
    global filename, com_sel
    user_filename = filename_entry.get()
    if(len(user_filename)>0):
        filename = user_filename + "_" + com_sel

# def user_entered_thresholds():
#     global ir_th_val, blue_th_val
#     ir_th_val = ir_th_entry.get()
#     blue_th_val = blue_th_entry.get()

def plot_start():
    global cond, ser1, com_sel, t_thread, stopEvent
    
    # user_entered_thresholds() # get user thresholds
    readData_init() # re-initialize the global variables for next run
    
    com_sel = tkvar.get() # get the selected com port
    if((len(com_sel)!=0) & (cond==False)): 
        cond = True
        user_entered_filename() # get user filename

        ser1 = open_serial_port(com_sel)
        if ser1.isOpen()==False: 
            ser1.open(); ser1.flushInput(); ser1.flushOutput(); 
        
        # serial_write(SASI_WR_TYPE_OP, SASI_WR_OP_STOP, 0.1) 
        # serial_write(SASI_WR_TYPE_THIR, int(ir_th_val), 0.1) 
        # serial_write(SASI_WR_TYPE_THBLUE, int(blue_th_val), 0.1) 
        # serial_write(SASI_WR_TYPE_OP, SASI_WR_OP_START, 0) 
        ser1.flushInput(); ser1.flushOutput();  # receive some message after write is send; flush for now

        print("----Start Measurement----")
        print("Time\t\t  Count\t  IR_LSB\t  BL_LSB\t  SmokeFlag\t dp\t MassCon")

        stopEvent = threading.Event()
        t_thread = threading.Thread(target=plotter, daemon=True) # 
        t_thread.start()
           
    else:
        print("Invalid COM port or already started!")

def plot_stop():
    global cond, ser1, filename, headers, t_thread, stopEvent
    if(cond == True): # prevent multiple clicks
        cond = False
        
        # print("Enter plot_stop")
        stopEvent.set() # set event to stop thread
        # t_thread.join()
        # root.after_cancel(AFTER_REF) # TTedit
        
        user_entered_filename() # check if user input a filename
        if Nrun>1: 
            save_data_txt(dd_all, headers, filename, fragment=0) # store multi-entries
        else:
            save_data_txt(dd_all, headers, filename, fragment=1) # store multi-entries
        save_fig(fig1, filename)

        # serial_write(SASI_WR_TYPE_OP, SASI_WR_OP_STOP, 0.1) # Stop the measurement 

        # close serial port gracefully
        if ser1.isOpen(): 
            ser1.flushInput(); ser1.flushOutput(); ser1.close() 
        print("----Stop Measurement----")
    

root = tk.Tk() # create window
root.title('Texas Instruments')
root.configure(background = 'white')
root.geometry("930x530+100+150") # set the window size, position of the window (0,0 at top left corner); "width_size x height_size + x_position + y_position"
# root.resizable(0,0)
# root.tk.call('tk', 'scaling', 2.0)
photo = tk.PhotoImage(file = '.\TI_logo.png')
root.iconphoto(False, photo)
# root.iconbitmap("TI_logo.ico")

fig1, (ax1, ax2) = plt.subplots(2,1) #figsize=(10,5), dpi=200
# ax1.set_title('')
ax1.set_xlabel('Time (s)')
ax1.set_ylabel('BL (LSB)')
#ax1.set_xlim(0, 100); ax1.set_ylim(0, 2050) # set limit
lines1 = ax1.plot([],[], lw=2, color='b', label='Blue')[0]
ax1.grid(); ax1.legend(loc='upper left')
ax1.yaxis.set_major_formatter(FormatStrFormatter('%.2f'))

ax11 = ax1.twinx()
ax11.set_ylabel('IR (LSB)')
lines2 = ax11.plot([],[], lw=2, color='r',label='IR')[0]
ax11.legend(loc='upper right')
ax11.yaxis.set_major_formatter(FormatStrFormatter('%.2f'))

ax2.set_xlabel('Time (s)')
ax2.set_ylabel('Particle size (um)')
lines3 = ax2.plot([],[], lw=2, color='b',label='dp')[0]
ax2.grid(); ax2.legend(loc='upper left')
ax2.yaxis.set_major_formatter(FormatStrFormatter('%.2f'))

ax22 = ax2.twinx()
ax22.set_ylabel('Mass Con. (mg/m\u00b3)')
lines4 = ax22.plot([],[], lw=2, color='k',label='MC')[0]
ax22.legend(loc='upper right')
ax22.yaxis.set_major_formatter(FormatStrFormatter('%.2f'))

plt.tight_layout()

canvas = FigureCanvasTkAgg(fig1, master=root)  # A tk.DrawingArea.
canvas.get_tk_widget().place(x=0, y=0, width=700, height=400)
canvas.draw()

### Create buttons 
# Start plot button
root.update()
start = tk.Button(root, text = "Start", font = ('calbiri',12),command = lambda: plot_start())
start.place(x=50, y=410)

# Stop plot button
root.update() 
stop = tk.Button(root, text = "Stop", font = ('calbiri',12), command = lambda:plot_stop())
stop.place(x=start.winfo_x()+start.winfo_reqwidth() + 20, y=start.winfo_y())

# Filename entry
root.update() 
filename_label = tk.Label(root, text = "Filename:", font = ('calbiri',12), bg="white")
filename_label.place(x=stop.winfo_x() + stop.winfo_reqwidth() + 20, 
                     y=stop.winfo_y() + abs((filename_label.winfo_reqheight() - stop.winfo_reqheight())/2))

root.update()
filename = "default_no_name"   # "no_smoke_room_ambient", "no_smoke_fl"
filename_entry = tk.Entry(root, font = ('calbiri',12)) 
filename_entry.place(x=filename_label.winfo_x()+filename_label.winfo_reqwidth() + 1, y=filename_label.winfo_y(), width=150)
filename_entry.insert(0, filename) # provide a default name

# # Threshold entries
# root.update()
# blue_th_val = 5   # Default value for blue smoke trigger threshold (LSB)
# blue_th_label = tk.Label(root, text = "Blue threshold (LSB):", font = ('calbiri',12), bg="white")
# blue_th_label.place(x=start.winfo_x(), y=start.winfo_y() + start.winfo_reqheight() + 8)
# root.update()
# blue_th_entry = tk.Entry(root, font = ('calbiri',12)) 
# blue_th_entry.place(x=blue_th_label.winfo_x() + blue_th_label.winfo_reqwidth() + 1, y=blue_th_label.winfo_y(), width=40)
# blue_th_entry.insert(0, blue_th_val) # provide a default value

# root.update()
# ir_th_val = 5   # Default value for ir smoke trigger threshold (LSB)
# ir_th_label = tk.Label(root, text = "IR threshold (LSB):", font = ('calbiri',12), bg="white")
# ir_th_label.place(x=blue_th_label.winfo_x(), y=blue_th_label.winfo_y() + blue_th_label.winfo_reqheight() + 8)
# root.update()
# ir_th_entry = tk.Entry(root, font = ('calbiri',12)) 
# ir_th_entry.place(x=ir_th_label.winfo_x() + ir_th_label.winfo_reqwidth() + 1, y=ir_th_label.winfo_y(), width=40)
# ir_th_entry.insert(0, ir_th_val) # provide a default value

root.update()
tkvar = tk.StringVar(root)
com_ports_list = get_com_port_id("XDS110 Class Application/User UART")
tkvar.set(com_ports_list[0])
cali11 = TkFont.Font(family='calbiri', size=11)
w = tk.OptionMenu(root, tkvar, *com_ports_list)
w.config(font=cali11)

# COM port selection entries
def change_dropdown(*args):
    tkvar.set('')
    w['menu'].delete(0, 'end')
    com_ports_list = get_com_port_id("XDS110 Class Application/User UART")
    for choice in com_ports_list:
        w['menu'].add_command(label=choice, command=tk._setit(tkvar, choice))
    tkvar.set(com_ports_list[0])

root.update()
refresh_button = tk.Button( root, text="Update COM", font=('calbiri',11), command = change_dropdown)
refresh_button.place(x=filename_entry.winfo_x()+filename_entry.winfo_reqwidth(), y=stop.winfo_y())
root.update()
w.place(x=filename_entry.winfo_x()+filename_entry.winfo_reqwidth(), y=refresh_button.winfo_y()+refresh_button.winfo_reqheight()+2)


# Stats plot
stat_label1 = tk.Label(root, text = "", font = ('calbiri',12), justify= tk.LEFT, bg="white")
stat_label1.place(x=720, y=10) # 720, 60
stat_label2 = tk.Label(root, text = "", font = ('calbiri',12), justify= tk.LEFT, bg="white")
stat_label2.place(x=720, y=160)

aq_label = tk.Label(root, text = "", font = ('calbiri',12), justify= tk.LEFT, bg="white")
aq_label.place(x=720, y=310)

smoke_alarm_label = tk.Label(root, text = "", font = ('calbiri',12), justify= tk.LEFT, bg="white")
smoke_alarm_label.place(x=720, y=360)

#ALC_label = tk.Label(root, text = "", font = ('calbiri',16), justify= tk.LEFT, bg="white")
#ALC_label.place(x=720, y=240)

smoke_label_blue = tk.Label(root, text = "", font = ('calbiri',14), justify= tk.LEFT, bg="white")
smoke_label_blue.place(x=640, y=360)

smoke_label_ir = tk.Label(root, text = "", font = ('calbiri',14), justify= tk.LEFT, bg="white")
smoke_label_ir.place(x=640, y=400)


######################################################################################

# root.mainloop() # Display the window
try:
    root.mainloop() # Display the window
    print("mainloop done, going to plot_stop")
    plot_stop()

except (Exception, KeyboardInterrupt) as e:
    print("Exception: ", e)
    plot_stop()

# %%
